@@ -115,11 +115,20 @@ class Git(Base):
         return Stash(self)
 
     @classmethod
-    def clone(cls, url, to_path, **kwargs):
+    def clone(
+        cls,
+        url: str,
+        to_path: str,
+        rev: Optional[str] = None,
+        **kwargs,
+    ):
         for _, backend in GitBackends.DEFAULT.items():
             try:
                 backend.clone(url, to_path, **kwargs)
-                return Git(to_path)
+                repo = cls(to_path)
+                if rev:
+                    repo.checkout(rev)
+                return repo
             except NotImplementedError:
                 pass
         raise NoGitBackendError("clone")
@@ -287,6 +296,7 @@ class Git(Base):
     add = partialmethod(_backend_func, "add")
     commit = partialmethod(_backend_func, "commit")
     checkout = partialmethod(_backend_func, "checkout")
+    fetch = partialmethod(_backend_func, "fetch")
     pull = partialmethod(_backend_func, "pull")
     push = partialmethod(_backend_func, "push")
     branch = partialmethod(_backend_func, "branch")
