@@ -49,7 +49,6 @@ class BaseGitBackend(ABC):
     def clone(
         url: str,
         to_path: str,
-        rev: Optional[str] = None,
         shallow_branch: Optional[str] = None,
         progress: Callable[["GitProgressEvent"], None] = None,
     ):
@@ -83,6 +82,15 @@ class BaseGitBackend(ABC):
     ):
         pass
 
+    @abstractmethod
+    def fetch(
+        self,
+        remote: Optional[str] = None,
+        force: bool = False,
+        unshallow: bool = False,
+    ):
+        pass
+
     @abstractmethod
     def pull(self, **kwargs):
         pass
