@@ -3,6 +3,7 @@ import locale
 import logging
 import os
 import stat
+from functools import partial
 from io import BytesIO, StringIO
 from typing import (
     TYPE_CHECKING,
@@ -18,13 +19,16 @@ from typing import (
 
 from funcy import cached_property
 
-from scmrepo.exceptions import AuthError, InvalidRemote, SCMError
+from scmrepo.exceptions import AuthError, CloneError, InvalidRemote, SCMError
+from scmrepo.progress import GitProgressReporter
 from scmrepo.utils import relpath
 
 from ...objects import GitObject
 from ..base import BaseGitBackend
 
 if TYPE_CHECKING:
+    from dulwich.repo import Repo
+
     from scmrepo.progress import GitProgressEvent
 
     from ...objects import GitCommit
@@ -76,6 +80,19 @@ class DulwichObject(GitObject):
         return self._sha
 
 
+class DulwichProgressReporter(GitProgressReporter):
+    """Dulwich progress reporter.
+
+    Works with both dulwich.porcelain methods which expect an 'errstream'
+    stream object and internal dulwich methods which expect a 'progress' stream
+    write method.
+    """
+
+    def write(self, msg: Union[str, bytes]) -> int:
+        self.__call__(msg)
+        return len(msg)
+
+
 class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
     """Dulwich Git backend."""
 
@@ -129,15 +146,69 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
     def root_dir(self) -> str:
         return self.repo.path
 
-    @staticmethod
+    @classmethod
     def clone(
+        cls,
         url: str,
         to_path: str,
-        rev: Optional[str] = None,
         shallow_branch: Optional[str] = None,
         progress: Callable[["GitProgressEvent"], None] = None,
     ):
-        raise NotImplementedError
+        from urllib.parse import urlparse
+
+        from dulwich.porcelain import NoneStream
+        from dulwich.porcelain import clone as git_clone
+
+        parsed = urlparse(url)
+        if os.name == "nt":
+            # NOTE: Dulwich client code doesn't handle mixed path seps when
+            # passed a file:// URL (i.e. file://C:\foo\bar) but we can just
+            # strip the prefix since Dulwich uses the same clone behavior
+            # for both paths and file:// URLs (as opposed to C-git)
+            if parsed.scheme == "file":
+                url = url[7:]
+        try:
+            clone_from = partial(
+                git_clone,
+                url,
+                target=to_path,
+                errstream=(
+                    DulwichProgressReporter(progress)
+                    if progress
+                    else NoneStream()
+                ),
+            )
+            if shallow_branch:
+                # NOTE: dulwich only supports shallow/depth for non-local
+                # clones. This differs from CLI git, where depth is used for
+                # file:// URLs but not direct local paths
+                if parsed.scheme in ("git", "git+ssh", "ssh", "http", "https"):
+                    depth = 1
+                else:
+                    depth = 0
+                repo = clone_from(
+                    depth=depth, branch=os.fsencode(shallow_branch)
+                )
+            else:
+                repo = clone_from()
+            cls._set_default_tracking_branch(repo)
+        except Exception as exc:
+            raise CloneError(url, to_path) from exc
+
+    @staticmethod
+    def _set_default_tracking_branch(repo: "Repo"):
+        from dulwich.refs import LOCAL_BRANCH_PREFIX, parse_symref_value
+
+        try:
+            ref = parse_symref_value(repo.refs.read_ref(b"HEAD"))
+        except ValueError:
+            return
+        if ref.startswith(LOCAL_BRANCH_PREFIX):
+            branch = ref[len(LOCAL_BRANCH_PREFIX) :]
+            config = repo.get_config()
+            section = ("branch", os.fsencode(branch))
+            config.set(section, b"remote", b"origin")
+            config.set(section, b"merge", ref)
 
     @staticmethod
     def init(path: str, bare: bool = False) -> None:
@@ -227,6 +298,23 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
     ):
         raise NotImplementedError
 
+    def fetch(
+        self,
+        remote: Optional[str] = None,
+        force: bool = False,
+        unshallow: bool = False,
+    ):
+        from dulwich.porcelain import fetch
+        from dulwich.protocol import DEPTH_INFINITE
+
+        remote_b = os.fsencode(remote) if remote else b"origin"
+        fetch(
+            self.repo,
+            remote_location=remote_b,
+            force=force,
+            depth=DEPTH_INFINITE if unshallow else None,
+        )
+
     def pull(self, **kwargs):
         raise NotImplementedError
 
@@ -451,13 +539,13 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
             return new_refs
 
         try:
-            from scmrepo.progress import GitProgressReporter
-
             client.send_pack(
                 path,
                 update_refs,
                 self.repo.object_store.generate_pack_data,
-                progress=GitProgressReporter(progress) if progress else None,
+                progress=(
+                    DulwichProgressReporter(progress) if progress else None
+                ),
             )
         except (NotGitRepository, SendPackError) as exc:
             raise SCMError("Git failed to push '{src}' to '{url}'") from exc
@@ -524,12 +612,10 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
                 f"'{url}' is not a valid Git remote or URL"
             ) from exc
 
-        from scmrepo.progress import GitProgressReporter
-
         fetch_result = client.fetch(
             path,
             self.repo,
-            progress=GitProgressReporter(progress) if progress else None,
+            progress=DulwichProgressReporter(progress) if progress else None,
             determine_wants=determine_wants,
         )
         for (lh, rh, _) in fetch_refs:
