@@ -143,7 +143,6 @@ class GitPythonBackend(BaseGitBackend):  # pylint:disable=abstract-method
     def clone(
         url: str,
         to_path: str,
-        rev: Optional[str] = None,
         shallow_branch: Optional[str] = None,
         progress: Callable[["GitProgressEvent"], None] = None,
     ):
@@ -189,19 +188,6 @@ class GitPythonBackend(BaseGitBackend):  # pylint:disable=abstract-method
         except GitCommandError as exc:  # pylint: disable=no-member
             raise CloneError(url, to_path) from exc
 
-        # NOTE: using our wrapper to make sure that env is fixed in __init__
-        repo = GitPythonBackend(to_path)
-
-        if rev:
-            try:
-                repo.checkout(rev)
-            except GitCommandError as exc:  # pylint: disable=no-member
-                raise RevError(
-                    "failed to access revision '{}' for repo '{}'".format(
-                        rev, url
-                    )
-                ) from exc
-
     @staticmethod
     def init(path: str, bare: bool = False) -> None:
         from funcy import retry
@@ -260,6 +246,24 @@ class GitPythonBackend(BaseGitBackend):  # pylint:disable=abstract-method
         else:
             self.repo.git.checkout(branch, force=force, **kwargs)
 
+    def fetch(
+        self,
+        remote: Optional[str] = None,
+        force: bool = False,
+        unshallow: bool = False,
+    ):
+        if not remote:
+            remote = "origin"
+        kwargs = {}
+        if force:
+            kwargs["force"] = True
+        if unshallow:
+            kwargs["unshallow"] = True
+        infos = self.repo.remote(name=remote).fetch(**kwargs)
+        for info in infos:
+            if info.flags & info.ERROR:
+                raise SCMError(f"fetch failed: {info.note}")
+
     def pull(self, **kwargs):
         infos = self.repo.remote().pull(**kwargs)
         for info in infos:
