@@ -17,7 +17,12 @@ from typing import (
 
 from funcy import cached_property
 
-from scmrepo.exceptions import MergeConflictError, RevError, SCMError
+from scmrepo.exceptions import (
+    CloneError,
+    MergeConflictError,
+    RevError,
+    SCMError,
+)
 from scmrepo.utils import relpath
 
 from ..objects import GitCommit, GitObject
@@ -150,11 +155,17 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
     def clone(
         url: str,
         to_path: str,
-        rev: Optional[str] = None,
         shallow_branch: Optional[str] = None,
         progress: Callable[["GitProgressEvent"], None] = None,
     ):
-        raise NotImplementedError
+        from pygit2 import GitError, clone_repository
+
+        if shallow_branch:
+            raise NotImplementedError
+        try:
+            clone_repository(url, to_path)
+        except GitError as exc:
+            raise CloneError(url, to_path) from exc
 
     @staticmethod
     def init(path: str, bare: bool = False) -> None:
@@ -204,6 +215,14 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
                 else:
                     self.repo.set_head(commit.id)
 
+    def fetch(
+        self,
+        remote: Optional[str] = None,
+        force: bool = False,
+        unshallow: bool = False,
+    ):
+        raise NotImplementedError
+
     def pull(self, **kwargs):
         raise NotImplementedError
 
@@ -232,7 +251,14 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
         raise NotImplementedError
 
     def active_branch(self) -> str:
-        raise NotImplementedError
+        if self.repo.head_is_detached:
+            raise SCMError("No active branch (detached HEAD)")
+        if self.repo.head_is_unborn:
+            # if HEAD points to a nonexistent branch we still return the
+            # branch name (without "refs/heads/" prefix) to match gitpython's
+            # behavior
+            return self.repo.references["HEAD"].target[11:]
+        return self.repo.head.shorthand
 
     def list_branches(self) -> Iterable[str]:
         raise NotImplementedError
@@ -577,18 +603,31 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
         msg: Optional[str] = None,
         squash: bool = False,
     ) -> Optional[str]:
-        from pygit2 import GIT_RESET_MIXED, GitError
+        from pygit2 import (
+            GIT_MERGE_ANALYSIS_FASTFORWARD,
+            GIT_MERGE_ANALYSIS_NONE,
+            GIT_MERGE_ANALYSIS_UNBORN,
+            GIT_MERGE_ANALYSIS_UP_TO_DATE,
+            GIT_MERGE_PREFERENCE_FASTFORWARD_ONLY,
+            GIT_MERGE_PREFERENCE_NO_FASTFORWARD,
+            GitError,
+        )
 
         if commit and squash:
             raise SCMError("Cannot merge with 'squash' and 'commit'")
 
-        if commit and not msg:
-            raise SCMError("Merge commit message is required")
-
         with self.release_odb_handles():
+            self.repo.index.read(False)
+            obj, _ref = self.repo.resolve_refish(rev)
+            analysis, ff_pref = self.repo.merge_analysis(obj.id)
+
+            if analysis == GIT_MERGE_ANALYSIS_NONE:
+                raise SCMError(f"'{rev}' cannot be merged into HEAD")
+            if analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE:
+                return None
+
             try:
-                self.repo.index.read(False)
-                self.repo.merge(rev)
+                self.repo.merge(obj.id)
                 self.repo.index.write()
             except GitError as exc:
                 raise SCMError("Merge failed") from exc
@@ -596,18 +635,54 @@ class Pygit2Backend(BaseGitBackend):  # pylint:disable=abstract-method
             if self.repo.index.conflicts:
                 raise MergeConflictError("Merge contained conflicts")
 
-            if commit:
-                user = self.default_signature
-                tree = self.repo.index.write_tree()
-                merge_commit = self.repo.create_commit(
-                    "HEAD", user, user, msg, tree, [self.repo.head.target, rev]
-                )
-                return str(merge_commit)
-            if squash:
-                self.repo.reset(self.repo.head.target, GIT_RESET_MIXED)
+            try:
+                if not (
+                    squash or ff_pref & GIT_MERGE_PREFERENCE_NO_FASTFORWARD
+                ):
+                    if analysis & GIT_MERGE_ANALYSIS_FASTFORWARD:
+                        return self._merge_ff(rev, obj)
+
+                    if analysis & GIT_MERGE_ANALYSIS_UNBORN:
+                        self.repo.set_head(obj.id)
+                        return str(obj.id)
+
+                if ff_pref & GIT_MERGE_PREFERENCE_FASTFORWARD_ONLY:
+                    raise SCMError("Cannot fast-forward HEAD to '{rev}'")
+
+                if commit:
+                    if not msg:
+                        raise SCMError("Merge commit message is required")
+                    user = self.default_signature
+                    tree = self.repo.index.write_tree()
+                    merge_commit = self.repo.create_commit(
+                        "HEAD",
+                        user,
+                        user,
+                        msg,
+                        tree,
+                        [self.repo.head.target, obj.id],
+                    )
+                    return str(merge_commit)
+
+                # --squash merge:
+                # HEAD is not moved and merge changes stay in index
+                return None
+            finally:
                 self.repo.state_cleanup()
                 self.repo.index.write()
-        return None
+
+    def _merge_ff(self, rev: str, obj) -> str:
+        if self.repo.head_is_detached:
+            self.repo.set_head(obj.id)
+        else:
+            branch = self.get_ref("HEAD", follow=False)
+            assert branch
+            self.set_ref(
+                branch,
+                str(obj.id),
+                message=f"merge {rev}: Fast-forward",
+            )
+        return str(obj.id)
 
     def validate_git_remote(self, url: str, **kwargs):
         raise NotImplementedError
